/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal;

import com.db4o.*;
import com.db4o.config.*;
import com.db4o.internal.marshall.MarshallerFamily;
import com.db4o.internal.marshall.ObjectHeaderAttributes;;


/**
 * @exclude
 */
final class CustomMarshallerFieldMetadata extends FieldMetadata {

	private final ObjectMarshaller _marshaller;
	
	public CustomMarshallerFieldMetadata(ClassMetadata containingClass, ObjectMarshaller marshaller) {
		super(containingClass, marshaller);
		_marshaller = marshaller;
	}
	
	public void calculateLengths(Transaction trans, ObjectHeaderAttributes header, Object obj) {
		header.addBaseLength(linkLength());
	}
	
    public void defragField(MarshallerFamily mf,ReaderPair readers) {
    	readers.incrementOffset(linkLength());
    }
	
    public void delete(MarshallerFamily mf, StatefulBuffer a_bytes, boolean isUpdate) {
    	incrementOffset(a_bytes);
    }
	
	public boolean hasIndex() {
		return false;
	}
	
	public Object getOrCreate(Transaction trans, Object onObject) {
		return onObject;
	}
	
	public void set(Object onObject, Object obj) {
		// do nothing
	}
	
	public void instantiate(MarshallerFamily mf, ObjectReference ref, Object onObject,
		StatefulBuffer reader) throws CorruptionException {
		_marshaller.readFields(onObject, reader._buffer, reader._offset);
		incrementOffset(reader);
	}
	
	public void marshall(ObjectReference yo, Object obj, MarshallerFamily mf,
		StatefulBuffer writer, Config4Class config, boolean isNew) {
		_marshaller.writeFields(obj, writer._buffer, writer._offset);
		incrementOffset(writer);
	}
	
	public int linkLength() {
		return _marshaller.marshalledFieldLength();
	}
	

}
