/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal.cs;

import java.io.*;
import com.db4o.*;
import com.db4o.foundation.*;
import com.db4o.foundation.network.*;
import com.db4o.internal.*;
import com.db4o.internal.cs.messages.*;

public final class ServerMessageDispatcher extends Thread {

    private String i_clientName;

    private boolean i_loggedin;
    private long i_lastClientMessage;

    private int i_pingAttempts = 0;
    private int i_nullMessages;
    private boolean i_sendCloseMessage = true;

    private final ObjectServerImpl i_server;

    private Socket4 i_socket;
    private ClientTransactionHandle _transactionHandle;
    
    private Hashtable4 _queryResults;
    
    private Config4Impl i_config;

    final int i_threadID;

    private final Object _lock = new Object();
    private final Object _mainLock;
    
    ServerMessageDispatcher(
        ObjectServerImpl aServer,
        ClientTransactionHandle transactionHandle,
        Socket4 aSocket,
        int aThreadID,
        boolean loggedIn,
        Object mainLock)
        throws Exception {
    	
    	_transactionHandle = transactionHandle;
    	_mainLock = mainLock;
    	
    	setDaemon(true);
        	
        i_loggedin = loggedIn;
         
        i_lastClientMessage = System.currentTimeMillis(); // don't start pinging from the start
        i_server = aServer;
		i_config = (Config4Impl)i_server.configure();
        i_threadID = aThreadID;
        setName("db4o message server " + aThreadID);
        try {
            i_socket = aSocket;
            i_socket.setSoTimeout(((Config4Impl)aServer.configure()).timeoutServerSocket());

            // TODO: Experiment with packetsize and noDelay
            // i_socket.setSendBufferSize(100);
            // i_socket.setTcpNoDelay(true);

        } catch (Exception e) {
            i_socket.close();
            throw (e);
        }
    }

    public void close() {
    	synchronized(_mainLock) {
	    	synchronized(_lock) {
		    	if (isClosed()) { 
		    		return;
		    	}    	
				_transactionHandle.releaseTransaction();
		        sendCloseMessage();
				_transactionHandle.close();
		        closeSocket();
		        removeFromServer();
	    	}
    	}
    }

	private void sendCloseMessage() {
		try {
            if (i_sendCloseMessage) {
                write(Msg.CLOSE);
            }
        } catch (Exception e) {
            if (Debug.atHome) {
                e.printStackTrace();
            }

        }
	}

	private void removeFromServer() {
		try {
            i_server.removeThread(this);
        } catch (Exception e) {
            if (Debug.atHome) {
                e.printStackTrace();
            }
        }
	}

	private void closeSocket() {
		try {
            i_socket.close();
        } catch (Exception e) {
            if (Debug.atHome) {
                e.printStackTrace();
            }
        }
        i_socket = null;
	}

    private boolean isClosed() {
		return i_socket == null;
	}

    Transaction getTransaction() {
    	return _transactionHandle.transaction();
    }

    public void run() {
        while (i_socket != null) {
            try {
                if(! messageProcessor()){
                    break;
                }
            } catch (Exception e) {
                if (_transactionHandle.isClosed()) {
                    break;
                }
                if(! i_socket.isConnected()){
                    break;
                }
                if (Deploy.debug) {
                    e.printStackTrace();
                }
                i_nullMessages++;
            }
            
            // TODO: Optimize - this doesn't need to be in the loop of executing statements

            if (i_nullMessages > 20 || pingClientTimeoutReached()) {
                if (i_pingAttempts > 5) {
                    // 
                    getStream().logMsg(33, i_clientName);
                    break;
                }
                if (null == i_socket) break;
                write(Msg.PING);
                i_pingAttempts++;
            }
        }
        close();
    }

	private boolean pingClientTimeoutReached() {
		return (System.currentTimeMillis() - i_lastClientMessage > i_config.timeoutPingClients());
	}
    
    private boolean messageProcessor() throws IOException{
        
        Msg message = Msg.readMessage(getTransaction(), i_socket);
        if(message == null){
            i_nullMessages ++;
            return true;
        }
        
        i_lastClientMessage = System.currentTimeMillis();
        i_nullMessages = 0;
        i_pingAttempts = 0;
        if (! i_loggedin) {
            if (Msg.LOGIN.equals(message)) {
                String userName = ((MsgD) message).readString();
                String password = ((MsgD) message).readString();
                User found = i_server.getUser(userName);
                if (found != null) {
                    if (found.password.equals(password)) {
                        i_clientName = userName;
                        LocalObjectContainer mainStream = _transactionHandle.mainStream();
                        mainStream.logMsg(32, i_clientName);
                        int blockSize = mainStream.blockSize();
                        int encrypt = mainStream.i_handlers.i_encrypt ? 1 : 0;
                        write(Msg.LOGIN_OK.getWriterForInts(getTransaction(), new int[] {blockSize, encrypt}));
                        i_loggedin= true;
                        setName("db4o server socket for client " + i_clientName);
                    } else {
                        write(Msg.FAILED);
                        return false;
                    }
                } else {
                    write(Msg.FAILED);
                    return false;
                }
            }
            return true;
        }
        
        if (message.processAtServer(this)) {
            return true;
        }
        
        processSpecialMsg(message);
        
        return true;
    }

    // Process speical messages.
	// TODO: will we use different class to differiate these messages like
	// MCommit, MRollback so that processSpeicalMsg method is not necessary at
	// all. All process code should be at processAtServer method.
	public boolean processSpecialMsg(Msg message) {
		if (Msg.PING.equals(message)) {
        	writeOK();
            return true;
        }
        
        if(Msg.OBJECTSET_FINALIZED.equals(message)){
        	int queryResultID = ((MsgD) message).readInt();
        	queryResultFinalized(queryResultID);
        	return true;
        }
        
        if (Msg.CLOSE.equals(message)) {
        	write(Msg.CLOSE);
            getTransaction().commit();
            i_sendCloseMessage = false;
            getStream().logMsg(34, i_clientName);
            return false;
        }
        
        if (Msg.IDENTITY.equals(message)) {
            respondInt((int)getStream().getID(getStream().identity()));
            return true;
        }
        
        if (Msg.CURRENT_VERSION.equals(message)){
            long ver = 0;
            synchronized(getStream()){
                ver = getStream().currentVersion();
            }
            write(Msg.ID_LIST.getWriterForLong(getTransaction(), ver));
            return true;
        }
        
        if (Msg.RAISE_VERSION.equals(message)) {
            long minimumVersion = ((MsgD)message).readLong();
            ObjectContainerBase stream = getStream();
            synchronized(stream){
                stream.raiseVersion(minimumVersion);
            }
            return true;
        } 
        
        if (Msg.GET_THREAD_ID.equals(message)) {
            respondInt(i_threadID);
            return true;
        }
        
        if (Msg.SWITCH_TO_FILE.equals(message)) {
            switchToFile(message);
            return true;
        }
        
        if (Msg.SWITCH_TO_MAIN_FILE.equals(message)) {
            switchToMainFile();
            return true;
        }
        
        if (Msg.USE_TRANSACTION.equals(message)) {
            useTransaction(message);
            return true;
        }
        return true;
	}

    private void writeOK() {
    	write(Msg.OK);
	}

	private void queryResultFinalized(int queryResultID) {
    	_queryResults.remove(queryResultID);
	}

	public void mapQueryResultToID(LazyClientObjectSetStub stub, int queryResultID) {
    	if(_queryResults == null){
    		_queryResults = new Hashtable4();
    	}
    	_queryResults.put(queryResultID, stub);
	}
	
	public LazyClientObjectSetStub queryResultForID(int queryResultID){
		return (LazyClientObjectSetStub) _queryResults.get(queryResultID);
	}

	private void switchToFile(Msg message) {
        synchronized (_mainLock) {
            String fileName = ((MsgD) message).readString();
            try {
                _transactionHandle.releaseTransaction();
            	_transactionHandle.acquireTransactionForFile(fileName);
                writeOK();
            } catch (Exception e) {
                if (Debug.atHome) {
                    System.out.println("Msg.SWITCH_TO_FILE failed.");
                    e.printStackTrace();
                }
                _transactionHandle.releaseTransaction();
                write(Msg.ERROR);
            }
        }
    }

    private void switchToMainFile() {
        synchronized (_mainLock) {
            _transactionHandle.releaseTransaction();
            writeOK();
        }
    }

    private void useTransaction(Msg message) {
        int threadID = ((MsgD) message).readInt();
		Transaction transToUse = i_server.findTransaction(threadID);
		_transactionHandle.transaction(transToUse);
    }
    
    private void respondInt(int response){
    	write(Msg.ID_LIST.getWriterForInt(getTransaction(), response));
    }
    
    public void write(Msg msg){
    	_transactionHandle.write(msg, i_socket);
    }
    
    public Socket4 socket(){
    	return i_socket;
    }
    
    private final LocalObjectContainer getStream() {
        return _transactionHandle.stream();
    }
    
}