/* Copyright (C) 2004 - 2007  db4objects Inc.  http://www.db4o.com

This file is part of the db4o open source object database.

db4o is free software; you can redistribute it and/or modify it under
the terms of version 2 of the GNU General Public License as published
by the Free Software Foundation and as clarified by db4objects' GPL 
interpretation policy, available at
http://www.db4o.com/about/company/legalpolicies/gplinterpretation/
Alternatively you can write to db4objects, Inc., 1900 S Norfolk Street,
Suite 350, San Mateo, CA 94403, USA.

db4o is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */
package com.db4o.internal.marshall;

import com.db4o.*;
import com.db4o.internal.*;
import com.db4o.internal.handlers.*;
import com.db4o.internal.query.processor.*;


class ArrayMarshaller1 extends ArrayMarshaller{
    
    public void calculateLengths(Transaction trans, ObjectHeaderAttributes header, ArrayHandler arrayHandler, Object obj, boolean topLevel){
        
        TypeHandler4 typeHandler = arrayHandler.i_handler;
        
        if(topLevel){
            header.addBaseLength(arrayHandler.linkLength());
        } else{
            header.addPayLoadLength(arrayHandler.linkLength());
        }

        if(typeHandler.hasFixedLength()){
            header.addPayLoadLength(arrayHandler.objectLength(obj));
        }else{
            header.addPayLoadLength(arrayHandler.ownLength(obj));
            Object[] all = arrayHandler.allElements(obj);
            for (int i = 0; i < all.length; i++) {
                typeHandler.calculateLengths(trans, header, false, all[i], true);
            }
        }
    }
    
    public void deleteEmbedded(ArrayHandler arrayHandler, StatefulBuffer reader) {
        
        int address = reader.readInt();
        reader.readInt();  // length
        if (address <= 0) {
            return;
        }
        
        int linkOffSet = reader._offset; 
        
        Transaction trans = reader.getTransaction();
        TypeHandler4 typeHandler = arrayHandler.i_handler;
        
        if (reader.cascadeDeletes() > 0 && typeHandler instanceof ClassMetadata) {
            reader._offset = address;
            if (Deploy.debug) {
                reader.readBegin(arrayHandler.identifier());
            }
            reader.setCascadeDeletes(reader.cascadeDeletes());
            for (int i = arrayHandler.elementCount(trans, reader); i > 0; i--) {
                arrayHandler.i_handler.deleteEmbedded(_family, reader);
            }
        }
        
        if(linkOffSet > 0){
            reader._offset = linkOffSet;
        }
    }
    
    public Object read(ArrayHandler arrayHandler,  StatefulBuffer reader) throws CorruptionException{
        int linkOffSet = reader.preparePayloadRead();
        Object array = arrayHandler.read1(_family, reader);
        reader._offset = linkOffSet;
        return array;
    }
    
    public void readCandidates(ArrayHandler arrayHandler, Buffer reader, QCandidates candidates) {
        reader._offset = reader.readInt();
        arrayHandler.read1Candidates(_family, reader, candidates);
    }
    
    public final Object readQuery(ArrayHandler arrayHandler, Transaction trans, Buffer reader) throws CorruptionException{
        reader._offset = reader.readInt();
        return arrayHandler.read1Query(trans,_family, reader);
    }
    
    public Object writeNew(ArrayHandler arrayHandler, Object obj, boolean restoreLinkOffset, StatefulBuffer writer) {
        if (obj == null) {
            writer.writeEmbeddedNull();
            return null;
        }
        int length = arrayHandler.objectLength(obj);
        int linkOffset = writer.reserveAndPointToPayLoadSlot(length);
        arrayHandler.writeNew1(obj, writer);
        if(restoreLinkOffset){
            writer._offset = linkOffset;
        }
        return obj;
    }

    protected Buffer prepareIDReader(Transaction trans,Buffer reader) {
        reader._offset = reader.readInt();
        return reader;
    }
    
    public void defragIDs(ArrayHandler arrayHandler,ReaderPair readers) {
    	int offset=readers.preparePayloadRead();
        arrayHandler.defrag1(_family, readers);
        readers.offset(offset);
    }
}
